@@ -32,19 +32,13 @@ if(mode=='vpn'):
         return output
 if(mode=='local'):
     def nsefetch(payload):
-        try: #Try the normal way.
+        try:
             output = requests.get(payload,headers=headers).json()
-        except: 
-            try: # Follow Praty's hack as mentioned here at https://forum.unofficed.com/t/solution-equity-history-function-doesnt-work/1197
-                output = requests.session().get(payload,headers=headers).json()
-
-            except: #Mimick FrontPage Visit
-                s=requests.session()
-
-                payload2 = "https://www.nseindia.com"
-                output2 = s.get(payload2,headers=headers).json()
-
-                output = s.get(payload,headers=headers).json()
+            print(output)
+        except ValueError:
+            s =requests.Session()
+            output = s.get("http://nseindia.com",headers=headers)
+            output = s.get(payload,headers=headers).json()
         return output
 
 
@@ -188,8 +182,12 @@ def nse_quote(symbol,section=""):
         return payload
 
 
-def nse_expirydetails(payload,i=0):
-    currentExpiry = payload['records']['expiryDates'][i]
+def nse_expirydetails(payload,i=0): #Can make problem. Use nse_expirydetails_by_symbol()
+
+    expiry_dates = payload['records']['expiryDates']
+    expiry_dates = [datetime.datetime.strptime(date, "%d-%b-%Y").date() for date in expiry_dates]
+    expiry_dates = [date.strftime("%d-%b-%Y") for date in expiry_dates if date >= datetime.datetime.now().date()]
+    currentExpiry=expiry_dates[i]    
     currentExpiry = datetime.datetime.strptime(currentExpiry,'%d-%b-%Y').date()  # converting json datetime to alice datetime
     date_today = run_time.strftime('%Y-%m-%d')  # required to remove hh:mm:ss
     date_today = datetime.datetime.strptime(date_today,'%Y-%m-%d').date()
@@ -208,29 +206,30 @@ def pcr(payload,inp='0'):
                 pass
     return pe_oi / ce_oi
 
+#forum.unofficed.com/t/unable-to-find-nse-quote-meta-api/702/4
+#Refer https://forum.unofficed.com/t/changed-the-nse-quote-ltp-function/1276
 def nse_quote_ltp(symbol,expiryDate="latest",optionType="-",strikePrice=0):
   payload = nse_quote(symbol)
-  #https://stackoverflow.com/questions/7961363/removing-duplicates-in-lists
-  #https://stackoverflow.com/questions/19199984/sort-a-list-in-python
-
-  #BankNIFTY and NIFTY has weekly options. Using this Jugaad which has primary base of assumption that Reliance will not step out of FNO.
-  #forum.unofficed.com/t/unable-to-find-nse-quote-meta-api/702/4
-  if((symbol in indices) and (optionType=="Fut")):
-    dates = expiry_list("RELIANCE","list")
-    if(expiryDate=="latest"): expiryDate=dates[0]
-    if(expiryDate=="next"): expiryDate=dates[1]
-
-  if(expiryDate=="latest") or (expiryDate=="next"):
-    dates=list(set((payload["expiryDates"])))
-    dates.sort(key = lambda date: datetime.datetime.strptime(date, '%d-%b-%Y'))
-    if(expiryDate=="latest"): expiryDate=dates[0]
-    if(expiryDate=="next"): expiryDate=dates[1]
 
   meta = "Options"
   if(optionType=="Fut"): meta = "Futures"
   if(optionType=="PE"):optionType="Put"
   if(optionType=="CE"):optionType="Call"
 
+  if(expiryDate=="latest") or (expiryDate=="next"):
+
+    if(meta=="Futures"):
+      selected_key = next((key for key in payload["expiryDatesByInstrument"] if "futures" in key.lower()), None)
+    if(meta=="Options"):
+      selected_key = next((key for key in payload["expiryDatesByInstrument"] if "options" in key.lower()), None)
+
+    expiry_dates=payload["expiryDatesByInstrument"][selected_key]
+    expiry_dates = [datetime.datetime.strptime(date, "%d-%b-%Y").date() for date in expiry_dates]
+    expiry_dates = [date.strftime("%d-%b-%Y") for date in expiry_dates if date >= datetime.datetime.now().date()]
+    if(expiryDate=="latest"): expiryDate=expiry_dates[0]
+    if(expiryDate=="next"): expiryDate=expiry_dates[1]
+  
+
   if(optionType!="-"):
       for i in payload['stocks']:
         if meta in i['metadata']['instrumentType']:
@@ -305,7 +304,10 @@ def nse_quote_meta(symbol,expiryDate="latest",optionType="-",strikePrice=0):
   return metadata
 
 def nse_optionchain_ltp(payload,strikePrice,optionType,inp=0,intent=""):
-    expiryDate=payload['records']['expiryDates'][inp]
+    expiry_dates = payload['records']['expiryDates']
+    expiry_dates = [datetime.datetime.strptime(date, "%d-%b-%Y").date() for date in expiry_dates]
+    expiry_dates = [date.strftime("%d-%b-%Y") for date in expiry_dates if date >= datetime.datetime.now().date()]
+    expiryDate=expiry_dates[inp]
     for x in range(len(payload['records']['data'])):
       if((payload['records']['data'][x]['strikePrice']==strikePrice) & (payload['records']['data'][x]['expiryDate']==expiryDate)):
           if(intent==""): return payload['records']['data'][x][optionType]['lastPrice']
@@ -608,13 +610,16 @@ def derivative_history_virgin(symbol,start_date,end_date,instrumentType,expiry_d
     instrumentType = instrumentType.lower()
 
     if(instrumentType=="options"):
-        if("NIFTY" in symbol): instrumentType="FUTSTK"
         instrumentType="OPTSTK"
-    if(instrumentType=="futures"):
         if("NIFTY" in symbol): instrumentType="OPTIDX"
-        instrumentType="FUTIDX"
+        
+    if(instrumentType=="futures"):
+        instrumentType="FUTSTK"
+        if("NIFTY" in symbol): instrumentType="FUTIDX"
+        
 
-    if(((instrumentType=="OPTIDX")or (instrumentType=="OPTSTK")) and (expiry_date!="")):
+    #if(((instrumentType=="OPTIDX")or (instrumentType=="OPTSTK")) and (expiry_date!="")):
+    if(strikePrice!=""):
         strikePrice = "%.2f" % strikePrice
         strikePrice = str(strikePrice)
 
@@ -623,7 +628,6 @@ def derivative_history_virgin(symbol,start_date,end_date,instrumentType,expiry_d
     logging.info(nsefetch_url)
     logging.info(payload)
     return pd.DataFrame.from_records(payload["data"])
-
 def derivative_history(symbol,start_date,end_date,instrumentType,expiry_date,strikePrice="",optionType=""):
     #We are getting the input in text. So it is being converted to Datetime object from String.
     start_date = datetime.datetime.strptime(start_date, "%d-%m-%Y")
@@ -682,11 +686,41 @@ def derivative_history(symbol,start_date,end_date,instrumentType,expiry_date,str
     return payload
 
 
-def expiry_history(symbol,start_date="",end_date=""):
+def expiry_history(symbol,start_date="",end_date="",type="options"):
     if(end_date==""):end_date=end_date
     nsefetch_url = "https://www.nseindia.com/api/historical/fo/derivatives/meta?&from="+start_date+"&to="+end_date+"&symbol="+symbol+""
     payload = nsefetch(nsefetch_url)
-    return payload['data'][2]
+
+    print(payload)
+
+    for key, value in payload['expiryDatesByInstrument'].items():
+      if type.lower() == "options" and "OPT" in key:
+          payload_data = payload['expiryDatesByInstrument'][key]
+          break
+      elif type.lower() == "futures" and "FUT" in key:
+          payload_data =  payload['expiryDatesByInstrument'][key]
+          break
+    
+    # Convert start_date and end_date to datetime objects
+    start_date = datetime.datetime.strptime(start_date, "%d-%m-%Y")
+    end_date = datetime.datetime.strptime(end_date, "%d-%m-%Y")
+
+    # Initialize an empty list to store filtered dates
+    filtered_date_payload = []
+
+    # Initialize a flag to check if the first date after end_date has been added
+    added_after_end_date = False    
+
+    # Iterate through date_payload and filter dates within the range
+    for date_str in payload_data:
+        date_obj = datetime.datetime.strptime(date_str, "%d-%b-%Y")
+        if start_date <= date_obj <= end_date:
+            filtered_date_payload.append(date_str)
+        elif date_obj > end_date and not added_after_end_date:
+            filtered_date_payload.append(date_str)
+            added_after_end_date = True
+    
+    return filtered_date_payload
 
 # # Nifty Indicies Site
 
@@ -874,4 +908,21 @@ def is_market_open(segment = "FO"): #COM,CD,CB,CMOT,COM,FO,IRD,MF,NDM,NTRP,SLBS
             return True
         if holiday['tradingDate'] == today_date:
             print(f"Market is closed today because of {holiday['description']}")
-            return False
\ No newline at end of file
+            return False
+
+def nse_expirydetails_by_symbol(symbol,meta ="Futures",i=0):
+    payload = nse_quote(symbol)
+
+    if(meta=="Futures"):
+      selected_key = next((key for key in payload["expiryDatesByInstrument"] if "futures" in key.lower()), None)
+    if(meta=="Options"):
+      selected_key = next((key for key in payload["expiryDatesByInstrument"] if "options" in key.lower()), None)
+
+    expiry_dates=payload["expiryDatesByInstrument"][selected_key]
+    expiry_dates = [datetime.datetime.strptime(date, "%d-%b-%Y").date() for date in expiry_dates]
+    expiry_dates = [date.strftime("%d-%b-%Y") for date in expiry_dates if date >= datetime.datetime.now().date()]
+    
+    currentExpiry=expiry_dates[i]
+    currentExpiry = datetime.datetime.strptime(currentExpiry,'%d-%b-%Y').date()    
+    dte = (currentExpiry - datetime.datetime.now().date()).days
+    return currentExpiry,dte
\ No newline at end of file
